import numpy as np
from time import sleep
from math import floor

from epics import caget, caput,PV

import iexcode.instruments.cfg as iex
from iexcode.instruments.IEX_BL_config import Beamline_Config
from iexcode.instruments.cameras import _enable_endstation_cameras

from iexcode.instruments.staff import staff_detector_dictionary
from iexcode.instruments.files_and_folders import check_run,make_user_folders,folder_mda
from iexcode.instruments.staff import staff_detector_dictionary
from iexcode.instruments.xrays import _xrays_detector_dictionary,_xrays_reset,xrays_get_all

from iexcode.instruments.conversions_constants import *
from iexcode.instruments.utilities import *
from iexcode.instruments.userCalcs import userStringSeq_clear, userStringSeq_pvs, userCalcOut_clear

from iexcode.instruments.Motors import Motors
from iexcode.instruments.current_amplifiers import SRS, ca_reset_all, _Keithley_pv
from iexcode.instruments.valves import branch_valve_close
from iexcode.instruments.shutters import branch_shutter_close

#branch specific
from iexcode.instruments.slits import slit3D_get
#endstation specific
from iexcode.instruments.kappa_det import Kappa_Detector
from iexcode.instruments.scalers import scaler_cts
from iexcode.instruments.MPA import MPA
from iexcode.instruments.spec_stuff import folders_spec

default_ioc = '29idKappa:'
#############################################################################
def Kappa_init(*userName,**kwargs):
    """
    used to intialize the Endstation class which carries scanRecord, logging ... parameters
    returns global detectors relavent to this endstation (tey,d3,d4,mesh,Kappa_scaler_pv,tth_pv,tthdet)

        *userName is an optional parameter, will be prompted if set_folders=True

        **kwargs:
            set_folders: sets the mda and EA folders (default => True)  
            reset: to reset the detectors in the IOC, etc (default => True)  
            xrays: sets global variable and detectors for x-ray (default => True) 
            mode: used sets the detectors 'user ' / 'staff' (default => 'user' )  
     """
    kwargs.setdefault('scan_ioc',default_ioc)
    kwargs.setdefault('xrays',True)
    kwargs.setdefault('BL_mode','user')
    kwargs.setdefault('set_folders',True)
    kwargs.setdefault('reset',True)

    #motors
    physical_motors = ['x','y','z','tth','kth','kap','kphi']
    psuedo_motors = ['th','chi','phi']
    global Kappa_Motors
    Kappa_Motors = Motors('Kappa',_Kappa_motor_dictionary(),physical_motors,psuedo_motors)

    #endstation
    iex.BL = Beamline_Config('Kappa',kwargs['scan_ioc'],kwargs['xrays'],kwargs['BL_mode'],Kappa_Motors)

    #setting folders
    if kwargs['set_folders']:
        if iex.BL.mode == 'staff':
            user_name = 'staff'
        else:
            if len(userName)==0:
                user_name = input('user name: ')
            else:
                user_name = userName[0]
        print(user_name)
        folders_Kappa(user_name,**kwargs)

        #update for default scanRecord advanced parameters
        iex.BL.mda.log('Kappa',user_name,_Kappa_log_header(),_Kappa_log_entries)

    #global detectors
    global tey,d3,d4,mesh,Kappa_scaler_pv,tth_pv,tthdet
    tey = SRS("29idMZ0:scaler1.S2", '29idd:A1')
    d3 = SRS("29idMZ0:scaler1.S3", '29idd:A2')
    d4 = SRS("29idMZ0:scaler1.S4", '29idd:A3')
    mesh = SRS("29idMZ0:scaler1.S14", '29idd:A4')
    Kappa_scaler_pv = '29idMZ0:scaler1.CNT'
    tth_pv = _Kappa_motor_dictionary('tth')[3]
    iex.mpa = MPA()
    tthdet = Kappa_Detector(Kappa_Motors)
    
    #resetting everything
    if kwargs['reset']:
        Kappa_reset(**kwargs)

    #enable cameras
    _enable_endstation_cameras()

    print ('Kappa initalized')
    #return any detectors or other parameters that you want access to from jupyter
    return tey,d3,d4,mesh,Kappa_scaler_pv,tth_pv,tthdet
##############################################################################################################
##############################                    detectors and motors                ##############################
##############################################################################################################
def _Kappa_detector_list():
    """
    list of detectors to trigger 

    Previously: part of Detector_List
    """
    ca_list=[]
    return ca_list


def _Kappa_detector_dictionary(**kwargs):
    """
    returns a dictionary of the default detectors

    **kwargs
    add_vortex: to specifiy to add the vortex detectors to the dictionary (True/False)

    Previously: Detector_Default
    """
    kwargs.setdefault('add_vortex',False)
    det_dict={}
    vortex={
        15:"",
        16:"",
        17:"",
    }
    sample_temp={
        23:"29idd:LS331:TC1:SampleA",
        24:"29idd:LS331:TC1:SampleB",
    }
    m3r={
        25:"29id_ps6:Stats1:CentroidX_RBV",
        26:"29id_ps6:Stats1:SigmaX_RBV",
        27:"29id_ps6:Stats1:CentroidTotal_RBV",
    }
    scalers={
        31:"29idMZ0:scaler1.S14",
        32:"29idMZ0:scaler1.S2",
        33:"29idMZ0:scaler1.S3",
        34:"29idMZ0:scaler1.S4",
        35:"29idMZ0:scaler1.S5",
        36:"29idMZ0:scaler1_calc1.B",
        37:"29idMZ0:scaler1_calc1.C",
        38:"29idMZ0:scaler1_calc1.D",
        39:"29idMZ0:scaler1_calc1.E",
    }
    mpa_dets={
        30:"29iddMPA:det1:TotalRate_RBV",
        41:"29iddMPA:Stats1:Total_RBV",
        42:"29iddMPA:Stats2:Total_RBV",
        43:"29iddMPA:Stats3:Total_RBV",
        44:"29iddMPA:Stats4:Total_RBV",
        45:"29iddMPA:Stats5:Total_RBV",
    }
    hkl={
        46:'<H>',
        47:'<K>',
        48:'<L>'
    } 
    motors={
        51:"29idKappa:m8.RBV",
        52:"29idKappa:m7.RBV",
        53:"29idKappa:m1.RBV",
        54:"29idKappa:m9.RBV",
        55:"29idKappa:Euler_ThetaRBV",
        56:"29idKappa:Euler_ChiRBV",
        57:"29idKappa:Euler_PhiRBV",
    }
    #hkl are listed just a place holders, they are filled in by thier scanning functions
    det_dict.update(sample_temp)
    det_dict.update(scalers)
    det_dict.update(mpa_dets)
    det_dict.update(motors)
    if kwargs['add_vortex']:
        det_dict.update(vortex)

    #add detectors related to beamline
    if iex.BL.xrays: 
        det_dict.update(_xrays_detector_dictionary())

    return det_dict


def _Kappa_motor_dictionary(name):
    """
    motor_dictionary = {name:[rbv,val,spmg,pv]} for physical and psuedo/Euler motors
    usage:
        KappaS_PVmotor('x') => ['29idKappa:m2.RBV', '29idKappa:m2.VAL', '29idKapp:m2.SPMG','29idKappa:m2']
    """
    motor_nums={
        'x':2,
        'y':3,
        'z':4,
        'tth':9,
        'kth':8,
        'kap':7,
        'kphi':1,
      }
    Euler_motors={
        'th':'29idKappa:Euler_Theta',
        'chi':'29idKappa:Euler_Chi',
        'phi':'29idKappa:Euler_Phi'
    }
    motor_dictionary = {}
    for name in motor_nums.keys():
        pv = '29idKappa:m'+str(motor_nums[name])
        motor_dictionary.update({name:[pv+'.VAL',pv+'.SPMG',pv]})    
    
    for name in Euler_motors.keys():
        pv = Euler_motors[name]
        motor_dictionary.update({name:[pv+'RBV',pv,'',pv]})
    
    return motor_dictionary

def Kappa_extra_pvs():
    """
    used to get the PV associated with a given pnuemonic

    """
    d={
        "TA":"29idd:LS331:TC1:Control",
        "TB":"29idd:LS331:TC1:SampleB",
        "det_nam":'29idKappa:userStringSeq6.STR1',
        "HV":'29idKappa:userCalcOut10.OVAL',
        "centroid":'29id_ps6:Stats1:CentroidX_RBV'        
    }
    return d

def Kappa_kth_offset_get():
    caget("29idKappa:userCalcOut1.G")

def Kappa_kth_offset_set(val):
    caput("29idKappa:userCalcOut1.G",val)


#############################################################################################################
##############################                 setting folder                   ##############################
##############################################################################################################
def folders_Kappa(user_name,**kwargs):
    """
    Create and sets (if create_only=False) all the folders the current run and ARPES user 
    Sets the FileName for logging to be "UserName/YYYYMMDD_log.txt using logname_Set()"
    
    **kwargs:
        set_folders = True (default); set the mda and EA scanRecords
                    = False; only makes the folders (was create_only)
        run: run cycle e.g. 2022_1 (default => check_run() )
        ftp = True / False (default); print what needs to be added to the ftps crontab and create additional ftp folders
        mda_ioc: will overwrite the default ioc for mda scans
        debug
    """
    kwargs.setdefault('set_folders',False)
    kwargs.setdefault('run',check_run())
    kwargs.setdefault('ftp',False)
    kwargs.setdefault('debug',False)

    run = kwargs['run']

    if kwargs['debug']:
        print("run,folder,user_name,ioc,ftp: ",run,iex.BL.folder,user_name,iex.BL.ioc,kwargs['ftp'])
    
    # Create User Folder:
    make_user_folders(run,iex.BL.folder,user_name,iex.BL.endstation,ftp=kwargs['ftp'])
    sleep(5)

    if kwargs["set_folders"]:
        # Set up MDA folder:
        folder_mda(run,iex.BL.folder,user_name,iex.BL.prefix,iex.BL.ioc)
        #logfile_name_set(iex.BL.endstation)
        #logfile_header(iex.BL.endstation,iex.BL.ioc,Kappa_log_header())


        # Set up SPEC folder:
        folders_spec(run,iex.BL.folder,user_name)

        # Set up MPA folder:
        #Folder_MPA(run,iex.BL.folder,user_name)

    #resetting
    if 'reset':
        Kappa_reset()

def Kappa_reset():
    """
    resets scanRecord, current amplifiers, mono limits and lakeshore
    """
    #writing default parameters to iex.BL.mda
    if iex.BL.mode=='staff':
        iex.BL.mda.detector_dictionary = staff_detector_dictionary()
    else:
         iex.BL.mda.detector_dictionary = _Kappa_detector_dictionary()
    iex.BL.mda.trigger_dictionary = _Kappa_trigger_dictionary()
    iex.BL.mda.scan_before_sequence = _Kappa_scan_before_sequence()
    iex.BL.mda.scan_after_sequence = _Kappa_scan_after_sequence()    
    
    #resetting the scanRecord
    print("resetting the scanRecord - "+iex.BL.ioc)
    iex.BL.mda.reset_all()

    #resetting the current amplifiers
    if iex.BL.xray:
        ca_reset_all()
    
    #resetting mono and anyother beamline stuff
    if iex.BL.xrays:
        _xrays_reset()

    #resetting mono and other beamline stuff
    if iex.BL.xrays:
        _xrays_reset()

    #motors home and sync
    _SmarAct_motors_home()
    _PI_motors_sync()
    Euler_motors_sync()


##############################################################################################################
##############################                    get all                 ##############################
##############################################################################################################
def Kappa_get_all(verbose=True):
    """
    returns a dictionary with the current status of the Kappa endstation and exit slit
    """
    vals = {}

    #sample postion
    motor_dictionary = _Kappa_motor_dictionary()
    for motor in motor_dictionary.keys():
        vals[motor]=Kappa_Motors.get(motor,verbose=False)

    #endstation/branch pvs
    extra_pvs = Kappa_extra_pvs()
    for key in extra_pvs.keys():
        vals.update(key,caget(extra_pvs[key]))
    if iex.BL.xrays:
        vals.update('exit_slit',slit3D_get())

    #beamline info
    if iex.BL.xray:
        beamline_info = xrays_get_all()
        vals.update(beamline_info) 

    mesh.get()
    vals.update({'mesh':mesh.current})

    if verbose:
        print("-----------------------------------------------------------")   
        for key in vals:
            print(key+" = "+vals[key])
        print("-----------------------------------------------------------")   
    return vals

##############################################################################################################
##############################                         logging                  ##############################
##############################################################################################################
def _Kappa_log_header():
    """
    header for the log file
    """
    h = "scan,motor,start,stop,step,x,y,z,tth,kth,kap,kphi,TA,TB,"
    h += "hv,exit_slit,GRT,ID_SP,ID_RBV,ID_Mode,ID_QP,TEY,mesh,det_name,mpa_HV,m3r_centroid,time,comment"
    header_list = {'Kappa':h}
    return header_list 

def _Kappa_log_entries():
    """
    enstation info for log file

    Previously: scanlog
    """
    vals = Kappa_get_all(verbose=False)
    x = vals['x']
    y = vals['y']
    z = vals['z']
    tth = vals['tth']
    kth = vals['kth']
    kap = vals['kap']
    kphi = vals['kphi']

    TA = vals['TA']
    TB = vals['TB']
    mesh.get()
    tey_current = tey.current
    mesh_current = mesh.current
    det_name = tthdet.name
    mpa_HV = iex.mpa.HV_get()
    m3r_centroid = vals['kphi']

    entry_list = ["x","y","z","tth","kth","kap","kphi","TA","TB","TEY","mesh","det_name","mpa_HV","m3r_centroid"]
    pv_list = [ x, y, z, tth, kth, kap,kphi, TA, TB,tey_current,mesh_current,det_name,mpa_HV,m3r_centroid]  
    format_list = [".2f",".2f",".2f",".2f",".2f",".2f",".2f",".2f",".2f","1.2e","1.2e","s",".0f",".0f"]

    return entry_list,pv_list,format_list

##############################################################################################################
##############################             Kappa scanRecord           ##############################
##############################################################################################################
def _Kappa_scan_before_sequence(scan_ioc,scan_dim,**kwargs):
    """
    writes the user string sequence to happen at the beginning of a scan
    returns before_scan_pv = pv for userStringSeq for before scan

    **kwargs
        seq_num: userStringSeq number in ioc => 9 (default)

    Previously: BeforeScan_StrSeq
    """
    kwargs.setdefault('seq_num',9)
    seq_num=kwargs['seq_num']

    before_scan_pv,before_scan_proc = userStringSeq_pvs(scan_ioc, seq_num)

    #clear and write the before scan user sequence
    userStringSeq_clear(scan_ioc,seq_num)
    caput(before_scan_pv+".DESC","Before Scan")

    #This is where you'd do something if need (CA -> 'Passive', etc)
    pv=''
    cmd=''
    caput(before_scan_pv+".LNK" +str(1),pv)
    caput(before_scan_pv+".STR" +str(1),cmd)

    return before_scan_proc

def _Kappa_scan_after_sequence(scan_ioc,scan_dim,**kwargs):
    """
    writes the user string sequence to happen at the end of a scan
    returns after_scan_pv = pv for userStringSeq for after scan

    **kwargs
        seq_num: userStringSeq number in ioc => 10 (default)
        snake: for snake scanning => False (default)

    Previously: AfterScan_StrSeq
    """
    kwargs.setdefault('seq_num',10)
    kwargs.setdefault('snake',False)
    seq_num=kwargs['seq_num']

    if 'scan_ioc' in kwargs:
        scan_ioc = kwargs['scan_ioc']
        try:
            iex.BL.ioc = kwargs['scan_ioc']
        except:
            error = 'undefined'
    else:
        scan_ioc = iex.BL.ioc

    after_scan_pv,after_scan_proc = userStringSeq_pvs(scan_ioc, seq_num)
    
    #clear and write the after scan user sequence
    userStringSeq_clear(scan_ioc,seq_num)
    caput(after_scan_pv+".DESC","After Scan")
        
    scan_pv = iex.BL.ioc+"scan"+str(scan_dim)
    ## Put scan record back in absolute mode
    caput(after_scan_pv+".LNK2",scan_pv+".P1AR")
    caput(after_scan_pv+".STR2","ABSOLUTE")

    ## Put Positionner Settling time to 0.1s
    caput(after_scan_pv+".LNK3",scan_pv+".PDLY NPP NMS")
    caput(after_scan_pv+".DO3",0.1)

    ## Clear DetTriggers 2 to 4:
    caput(after_scan_pv+".LNK4",scan_pv+".T2PV NPP NMS")    #remove trigger 2 (used for EA) after scan

    if kwargs['snake']:
        snake_dim = scan_dim - 1
        #snake_proc= Kappa_snake_pv(snake_dim,enable=True)
        #caput(after_scan_pv+".LNK10",Kappa_snake_pv()+"PP NMS")
        #caput(after_scan_pv+".D10",1)

    return after_scan_proc

def _Kappa_detector_triggers_sequence(scan_ioc,scan_dim,**kwargs):    # do we need to add 29idb:ca5 ???
    """
    """
    kwargs.setdefault(seq_num,8)
    seq_num=kwargs['seq_num']

    detector_triggers_pv,detector_triggers_proc = userStringSeq_pvs(iex.BL.ioc, seq_num)
       
    #clear the userStringSeq
    userStringSeq_clear(scan_ioc,seq_num=kwargs['seq_num'])
    caput(detector_triggers_pv+".DESC","Kappa_Trigger1")

    scaler_pv = Kappa_scaler_pv()

    caput(detector_triggers_pv+".LNK" +str(1),scaler_pv)
    caput(detector_triggers_pv+".WAIT"+str(1),"After"+str(last))
    
    ca_list = _Kappa_detector_list()
    last = len(ca_list)
    for i,ca in enumerate(ca_list):
        ca_pv = _Keithley_pv(ca[0], ca[1])+':read.PROC CA NMS'
        caput(detector_triggers_pv+".LNK" +str(i+2),ca_pv)
        caput(detector_triggers_pv+".WAIT"+str(i+2),"After"+str(last))

    return detector_triggers_proc

def _Kappa_trigger_dictionary(scan_ioc,scan_dim,**kwargs):
    """
    need to do something
    """
    trigger_dictionary = {
        1:_Kappa_detector_triggers_sequence(scan_ioc,scan_dim,**kwargs),
    }
    return trigger_dictionary




##############################################################################################################
##############################        Kappa  Motor Scan Set Up        ##############################
##############################################################################################################

def Kappa_sample_Euler_list():
    """
    returns list of motor names used by mvsample
    """
    motor_list = ['th','chi','phi']
    return motor_list

def Kappa_4c_mprint():
    """
    returns the dictionary of the current sample position in 4c units 
    
    motors=['th','chi','phi']

    Previously: diffracto?
    """
    positions={}
    motors=Kappa_sample_Euler_list()
    for motor in motors:
        positions.update(motor,_Kappa_motor_dictionary()(motor)[0])
    return positions

def Kappa_4c_move(th_chi_phi_list):
    """
    moves the sample in 4c (Euler) units
    Previously: mv4C
    """
    motor_list=Kappa_sample_Euler_list()
    for motor,i in enumerate(motor_list):
        Kappa_Motors.move(motor,motor_list[i],wait=True,verbose=False)
    
    for motor,i in enumerate(motor_list):
        Kappa_Motors.get(motor,verbose=True)

def _SmarAct_motor_list():
    """
    returns a list of the motor names for the SmarAct motors in the Kappa
    """
    return ['x','y','z']

def _SmarAct_motors_home():
    """
    Homes the piezo (x,y,z). Home position is middle of travel

    Previously: Home_SmarAct_Motor
    """
    motor_dictionary = _Kappa_motor_dictionary()
    for motor in _SmarAct_motor_list():
        pv = motor_dictionary[motor][3]
        caput(pv+'.HOMF',1)
        sleep(10)
    print('SamrAct motors VAL homed')

def _SmarAct_enable():
    motor_dictionary = _Kappa_motor_dictionary()
    for motor in _SmarAct_motor_list():
        spmg = motor_dictionary[motor][2]
        caput(spmg,3)  # 3=Go

def _SmarAct_disable():
    motor_dictionary = _Kappa_motor_dictionary()
    for motor in _SmarAct_motor_list():
        spmg = motor_dictionary[motor][2]
        caput(spmg,0)  # 1=Stop

def _PI_motor_list():
    """
    returns a list of the motor names for the SmarAct motors in the Kappa
    """
    return ['kap','kth','tth']

def _PI_motors_sync():
    motor_dictionary = _Kappa_motor_dictionary()
    for motor in _PI_motor_list():
        val_pv = motor_dictionary[motor][1]
        rbv_pv = motor_dictionary[motor][0]
        current_rbv=caget(rbv_pv)
        caput(val_pv,current_rbv)
    print('PI motors VAL synced to RBV')

def Euler_motors_sync():
    """
    Syncs the Euler motores

    Previously: Sync_Euler_Motor
    """
    caput('29idKappa:Kappa_sync.PROC',1)
    sleep(1)
    caput('29idKappa:Kappa_sync.PROC',1)
    print('Euler motors VAL/RBV synced to physical motors')

def Kappa_kphi_reset_zero(val):
    """
    resets the zero for the kphi motor

    """
    kphi_pv = _Kappa_motor_dictionary()['kphi'][3]
    caput(kphi_pv+".SET",1)    # 1 = Set
    sleep(0.5)
    caput("kphi_pv.VAL",val)
    sleep(0.5)
    caput(kphi_pv+".SET",0)    # 0 = Use
    print("\nkphi has been reset to " +str(val))

def Kappa_tth_reset_zero():
    """
    resets the zero for the tth motor
    """
    tthdet.tth0_set()

def th2th_scan_sensitivity(th_table,gain_num_table,gain_unit_table,**kwargs):
    """
    Scans th2th with variable gain on SRS# (srs_num).
    Gain is specified for a given th2th range by building tables as follow:

    th_table = np.array([])
    gain_num_table = np.array([])
    gain_unit_table = np.array([])

    Example:
        for i in RangeUp(10,20,0.5):
            th_table = np.append(th_table, i)
            gn_table = np.append(gn_table, gain_dict(2))
            gu_table = np.append(gu_table, unit_dict('mA'))
        for i in RangeUp(20.5,40,0.5):
            th_table = np.append(th_table, i)
            gn_table = np.append(gn_table, gain_dict(1))
            gu_table = np.append(gu_table, unit_dict('uA'))
        for i in RangeUp(40.5,60,0.5):
            th_table = np.append(th_table, i)
            gn_table = np.append(gn_table, gain_dict(10))
            gu_table = np.append(gu_table, unit_dict('pA'))
    
    kwargs:
        detector (default tey)
        scan_dim: 1  (default)
        cts: integration time for scalers and mpa/mcp => 0.1 (default)
        execute: True/False to start the scan => True (default)
       
    Previoulsy: Scan_th2th_sensitivity
    """
    kwargs.setdefault('cts',0.1)
    kwargs.setdefault('execute',True)

    if 'detector' not in kwargs:
        detector = tey
    else:
        detector = kwargs['detector']

    kth_offset = Kappa_kth_offset_get()

    motor_dictionary = _Kappa_motor_dictionary()
    kth_val,kth_rbv,kth_spmg,kth_pv = motor_dictionary['kth']
    tth_val,tth_rbv,tth_spmg,tth_pv = motor_dictionary['tth']

    detector_pv = detector._srs_pv
    gain_num_pv = detector_pv + "sens_num.VAL"
    gain_unit_pv= detector_pv + "sens_unit.VAL"

    kth_table = th_table + kth_offset
    tth_table = 2*th_table

    kwargs.update("positioner_num",1)
    iex.BL.mda.fillin_table(kth_val,kth_rbv,kth_table,**kwargs)

    kwargs.update("positioner_num",2)
    iex.BL.mda.fillin_table(tth_val,tth_rbv,tth_table,**kwargs)

    kwargs.update("positioner_num",3)
    iex.BL.mda.fillin_table(gain_num_pv,"",gain_num_table,**kwargs)

    kwargs.update("positioner_num",4)
    iex.BL.mda.fillin_table(gain_unit_pv,"",gain_unit_table,**kwargs)

    print("\nDon't forget to clear extra positionners at the end of the scan if you were to abort the script using the function:")
    print("                Clear_Scan_Positioners('Kappa',1)")
 
    scaler_cts(kwargs['cts'],verbose=False)
    if kwargs['execute']:
        iex.BL.mda.go(**kwargs)

        #clean up after scan
        iex.BL.mda.table_reset_after(**kwargs)
        scaler_cts(verbose=False)

def scan_th2th(tth_start,tth_stop,tth_step,th_offset,**kwargs):
    """
    Used for a linear (not table) scan where th =  tth /2 + th_offset
    **kwargs
        scan_dim: 1  (default)
        cts: integration time for scalers and mpa/mcp => 0.1 (default) 
        execute: True/False to start the scan => True (default)

    Previously: scanth2th
    """
    kwargs.setdefault('scan_dim',1)
    kwargs.setdefault('cts',0.1)
    kwargs.setdefault('execute',True)
    
    th_start = (tth_start)/2+th_offset
    th_stop = (tth_stop)/2+th_offset
    th_step = int(tth_step/2)
    
    print('tth: '+str(tth_start)+"/"+str(tth_stop)+"/"+str(tth_step))
    print('th: '+str(th_start)+"/"+str(th_stop)+"/"+str(tth_step/2.0))
    
    #write to the scanRecord
    th_val,th_rbv,th_spmg,th_pv = _Kappa_motor_dictionary['th']
    tth_val,tth_rbv,tth_spmg,tth_pv = _Kappa_motor_dictionary['tth']

    kwargs.update("positioner_num",2)
    iex.BL.mda.fillin(th_val,th_rbv,th_start,th_stop,th_step,**kwargs)
    
    kwargs.update("positioner_num",1)
    iex.BL.mda.fillin(tth_val,tth_rbv,tth_start,tth_stop,tth_step,**kwargs)
    
    scaler_cts(kwargs['cts'],verbose=False)
    if kwargs['execute']:
        iex.BL.mda.go(**kwargs)

        #clean up after scan
        scaler_cts(verbose=False)


def th2th_table(tth_table,th0,**kwargs):
    """
    Create a table for tth, e.g.:
        mytable_tth=[]
        for q in range(2,41,1):
            mytable_tth.append(q)
        print(mytable_tth)
        
    th0 = th value at grazing incidence
    cts = counting time (default is 1s)
    
    **kwargs
        scan_dim: 1  (default)
        cts: integration time for scalers and mpa/mcp => 0.1 (default) 
        execute: True/False to start the scan => True (default)

    Previously: scanth2th_table

    """
    kwargs.setdefault('cts',0.1)
    kwargs.setdefault('execute',True)
  
    tth_table=np.asarray(tth_table)
    th_table=tth_table/2.0+th0

    th_val,th_rbv,th_spmg,th_pv = _Kappa_motor_dictionary['th']
    tth_val,tth_rbv,tth_spmg,tth_pv = _Kappa_motor_dictionary['tth']

    #write to the scanRecord
    kwargs.update("positioner_num",1)
    iex.BL.mda.fillin_table(tth_val,tth_rbv,tth_table,**kwargs)

    kwargs.update("positioner_num",2)
    iex.BL.mda.fillin_table(th_val,th_rbv,th_table,**kwargs)
    iex.BL.mda.positioner_after_scan(after="STAY")


    scaler_cts(kwargs['cts'],verbose=False,**kwargs)
    if kwargs['execute']:
        iex.BL.mda.go(**kwargs)

        #clean up after scan
        iex.BL.mda.table_reset_after()
        scaler_cts(verbose=False)
        iex.BL.mda.positioner_after_scan(after="PRIOR POS",**kwargs)




#############################################################################################################
##############################             Preset Positions        ##############################
##############################################################################################################
def _KappaTransfer_StrSeq():
    #User= [    DESC,        x,        y,      z,        tth,       kth,       kap,    kphi]
    User = ["Kappa Transfer",0, -2650, -650, 0, 57, 0, -88]
    n=4
    _KappaPreset_StrSeq(n,User)


def _KappaGrazing_StrSeq(): #Need to determine positions and then add to the Kappa graphic
    #Dial= [    DESC,        x,        y,      z,        tth,       kth,       kap,    kphi]
    User = ["Kappa Grazing",0, 0, 0, 0, 57.045, 134.76,57.045]
    n=3
    _KappaPreset_StrSeq(n,User)

def Kappa_ResetPreset():
    _KappaGrazing_StrSeq()
    _KappaTransfer_StrSeq()










##############################################################################################################
##############################             Kappa light       ##############################
############################################################################################################## 
def Kappa_light(ON_OFF):
    """
    Previously:light
    """
    light_pv = '29idd:Unidig1Bo0'
    if ON_OFF.lower() == 'on':
        light=0
    elif ON_OFF.lower() == 'off':
        light=1
    caput(light_pv,light)
    print(("Turning light "+ON_OFF+"."))


##############################################################################################################
########################        Scan Temp and Pressure           ##############################
##############################################################################################################
def Kappa_temperature_pressure_scan(scan_dim=1):
    """
    starts scan to monitor temperature and pressure in the kappa chamber

    Previously: Kappa_ScanTempPres
    """

    pv="29id"+iex.BL.mda.ioc+":scan"+str(scan_dim)
    #Clear all scan pvs
    caput(pv+".CMND",6)
    #Set detectors
    caput(pv+".D01PV","29idd:tc1:getVal_A.VAL")
    caput(pv+".D02PV","29idd:tc1:getVal_B.VAL")
    caput(pv+".D03PV","29idb:VS11D.VAL")
    #time scan
    iex.BL.mda.time_go()



##############################################################################################################
##############################             Kappa safestate        ##############################
##############################################################################################################
def Kappa_safe_state(**kwargs):
    """
    puts the C-branch in a safe state, 
    **kwargs
        MPA_off = True; turns off EA HV
        shutter_close = True; closes the D-shutter
        valve_close = True; closes the D-valve
    """
    kwargs.setdefault("MPA_off",True)
    kwargs.setdefault("shutter_close",True)
    kwargs.setdefault("valve_close",True)
    
    if kwargs["mpa_off"]:
        try:
            iex.mpa.HV_off()
        except:
            print('MPA is not running')

    if kwargs['shutter_close']:
        branch_shutter_close()
    
    if kwargs['valve_close']:
       branch_valve_close()


    