from time import sleep
from os import mkdir
from os.path import exists

from epics import caget,caput

from iexcode.instruments.files_and_folders import get_next_fileNumber, check_run
from iexcode.instruments.scanRecord import *

##############################################################################################################
##############################             Scan Bakeout            ##############################
##############################################################################################################

def bakout_scan(scan_ioc_name,chamber):
    """
    sets up the scan record to record temperature  and pressures

    scan_ioc_name = 'ARPES', 'Kappa','Test'
        chamber = "Beamline"; Yokawgawa + all beamline ion pumps and vacuum gauges
        chamber = "Mono"; Beamline + mono thermocouples
        chamber = "ARPES"; Beamline + ARPES pressures and temperatures
        chamber = "Kappa"; Beamline + Kappa pressures and temperatures
        chamber = "Stinger"; Stinger temperatures + ARPES and Kappa temperatures and pressures

    Previously: Scan_Bakeout
    """
    #setting the folders and save datat
    run = check_run()
    folder='/net/s29data/export/data_29idb/Bakeout/'
    subfolder='/'+chamber+'/'+run
    prefix='mda_'

    scan_ioc = '29id'+scan_ioc_name+":"
    
    caput(scan_ioc+'saveData_fileSystem',folder)
    caput(scan_ioc+'saveData_subDir',subfolder)
    caput(scan_ioc+'saveData_baseName',prefix)

    MyPath="/net/s29data/export/data_29idb/Bakeout/"+chamber+"/"+run
    print("\nBakeout folder: " + MyPath)
    if not (exists(MyPath)):
        mkdir(MyPath)
        sleep(1)
        FileNumber=1
    else:
        FileNumber=get_next_fileNumber(MyPath,prefix[:-1])
    caput(scan_ioc+'saveData_scanNumber',FileNumber)
    
    sleep(5)
    SaveStatus=caget(scan_ioc+'saveData_status',as_string=True)
    SaveMessage=caget(scan_ioc+'saveData_message',as_string=True)
    print("\nSave Status "+scan_ioc+" "+SaveStatus+" - "+SaveMessage)

    #setting up scan record
    detector_dictionary = bakeout_detector_dictionary(chamber)

    #caput("29idb:ca1:read.SCAN",0)  
    #trigger_dictionary = {1:"29idb:ca1:read"}
    trigger_dictionary = {1:""}
    
    before_scan_pv = ""
    after_scan_pv = ""

    mda_scan = ScanRecord(scan_ioc,detector_dictionary,trigger_dictionary,before_scan_pv,after_scan_pv)

    mda_scan.time_scan(99999,60)



def bakeout_detector_dictionary(chamber):    
    """
    returns a dictionary of the default detectors

    """
    thermocouples={
    1:"29iddau1:dau1:001:ADC",
    2:"29iddau1:dau1:002:ADC",
    3:"29iddau1:dau1:003:ADC",
    4:"29iddau1:dau1:004:ADC",
    6:"29iddau1:dau1:006:ADC",
    7:"29iddau1:dau1:007:ADC",
    8:"29iddau1:dau1:008:ADC",
    9:"29iddau1:dau1:009:ADC",
    10:"29iddau1:dau1:010:ADC",
    }
    ion_gauges={  
    21:"29idb:VS1A.VAL",
    22:"29idb:VS2A.VAL",
    23:"29idb:VS3AB.VAL",
    24:"29idb:VS4B.VAL",
    25:"29idb:VS5B.VAL",
    26:"29idb:VS6B.VAL",
    27:"29idb:VS7B.VAL",
    28:"29idb:VS8C.VAL",
    29:"29idb:VS8CSP.VAL",
    30:"29idb:VS9C.VAL",
    31:"29idb:VS10C.VAL",
    32:"29idb:VS8D.VAL",
    33:"29idb:VS9D.VAL",
    34:"29idb:VS10D.VAL",
    }
    ion_pumps={
    41:"29idb:IP1A.VAL",
    42:"29idb:IP2A.VAL",
    43:"29idb:IP3A.VAL",
    44:"29idb:IP3B.VAL",
    45:"29idb:IP4B.VAL",
    46:"29idb:IP5B.VAL",
    47:"29idb:IP6B.VAL",
    48:"29idb:IP7B.VAL",
    49:"29idb:IP7C.VAL",
    51:"29idb:IP8C1.VAL",
    52:"29idb:IP8C2.VAL",
    53:"29idb:IP9C.VAL",
    54:"29idb:IP10C1.VAL",
    55:"29idb:IP10C2.VAL",
    56:"29idb:IP7D.VAL",
    57:"29idb:IP8D1.VAL",
    58:"29idb:IP8D2.VAL",
    59:"29idb:IP9D.VAL",
    60:"29idb:IP10D1.VAL",
    61:"29idb:IP10D2.VAL",
    }

    mono = {
    11:"29idmono:TC1_MON",
    12:"29idmono:TC2_MON",
    13:"29idmono:TC3_MON",
    14:"29idmono:TC4_MON",
    15:"29idmono:TC5_MON",
    16:"29idmono:TC6_MON",
    }

    ARPES = {
    11:"29idc:VS11C.VAL",
    12:"29idc:VSCUBE.VAL",
    13:"29idc:IP11C1.VAL",
    14:"29idc:IP11C2.VAL",
    15:"29idc:IPCUBE.VAL",
    16:"29idARPES:LS335:TC1:IN1",
    17:"29idARPES:LS335:TC1:IN2",
    }

    Kappa = {
    11:"29idb:VS11D.VAL",
    12:"29idb:VS12D.VAL",
    13:"29idd:LS331:TC1:SampleA",
    14:"29idd:LS331:TC1:SampleB",
    }

    Stinger = {
    1:"29idc:LS340:TC2:Control",
    2:"29idc:LS340:TC2:Sample",

    3:"29idARPES:LS335:TC1:IN1",
    4:"29idARPES:LS335:TC1:IN2",

    5:"29idd:LS331:TC1:SampleA",
    6:"29idd:LS331:TC1:SampleB",

    7:"29idc:VS11C.VAL",
    8:"29idc:IP11C1.VAL",
    9:"29idc:IP11C2.VAL",

    10:"29idb:VS11D.VAL",
    }
    d={}
    if chamber.lower() == "Beamline".lower():
        d.update(thermocouples)
        d.update(ion_gauges)
        d.update(ion_pumps)
        print("ScanRecord ready")

    if chamber.lower() == "Mono".lower():
        d.update(thermocouples)
        d.update(ion_gauges)
        d.update(ion_pumps)
        d.update(mono)
        print('Setting up Mono temperatures')

    if chamber.lower() == "ARPES".lower():
        d.update(ARPES)
        print('Setting up ARPES temperatures')
        
    if chamber.lower() == "Kappa".lower():
        d.update(Kappa)
        print('Setting up Kappa temperatures')

    if chamber.lower() == "Stinger".lower():
        d.update(Stinger)
        print('Setting up Stinger temperatures')

    return d
