
import time 
from epics import caget,caput,PV

import iexcode.instruments.cfg as iex
from iexcode.instruments.AD_utilities import *

##############################################################################################################
##############################              Camera PV dictionary            ##############################
##############################################################################################################

def _pv_dictionary():
    """
    dictionary of pv names for the beamline cameras
    """
    d={
        1:"29id_ps1:",
        2:"29id_ps2:",
        3:"29id_ps3:",
        4:"29id_ps4:",
        5:"29idarv5:",
        6:"29id_ps6:",
    }
    return d

##############################################################################################################
##############################             Endstation Cameras           ##############################
##############################################################################################################
   
def _enable_endstation_cameras(unused_off=True):
    """
    Turns on cameras related to a given endstation if they are connected
    unused_off=True turns off other cameras

    """
    endstation=iex.BL.endstation

    #For new endstation modify here:
    cam_dict={'ARPES':[0,1,2],'Kappa':[3,4,6]}  # index of cam_list 

    d=_pv_dictionary()
    for num in d.keys(): 
        pv=d[num]+"cam1:Acquire"   
        #camera in list
        if num in cam_dict[endstation]:
            #two connected to deal with network traffic
            PV(pv).connected
            time.sleep(0.1)
            if PV(pv).connected:  
                caput(pv,'Acquire')
            else:
                print(d[num]+" ioc in not running")
        #camera not in list
        else:
            if unused_off:
                if PV(pv).connected:  
                    caput(pv,'Done')


##############################################################################################################
##############################             Camera Snapping an image         ##############################
##############################################################################################################

def cam_snap(cam_num,ADtype='TIFF',**kwargs):
    """
    takes a camera image and saves it in the user folder
    
    **kwargs:
    ExposureTime: changes both the exposure time and the acquire time for the snapshot
                    resets after acquisition
    FreeRun: True => disable setting and go back to continuous acquision 
                False => leave saving enabled and camera in single acquision

    """
    ADplugin = _pv_dictionary()[cam_num]+ADtype+":"
    AD_SaveFileSetup(ADplugin,iex.BL.mda,**kwargs)
    AD_snap(ADplugin,**kwargs)




##############################################################################################################
##############################             Scanning with a camera           ##############################
##############################################################################################################
def cam_scan_setup(cam_num,ADtype='TIFF',**kwargs):
    """
    sets up the BL scanRecord to trigger the camera (trigger - 2)
    """
    ADplugin = _pv_dictionary()[cam_num]+ADtype+":"
    ADplugin_ScanSetup(ADplugin,iex.BL.mda, **kwargs)  

def cam_live(cam_num,ADtype='TIFF',**kwargs):
    """
    puts camera in no save and continuous
    a.k.a free run

    """
    ADplugin = _pv_dictionary()[cam_num]+ADtype+":"
    AD_FreeRun(ADplugin,**kwargs)