import numpy as np

from epics import caget

##############################################################################################################
##############################            conversions         ##############################
##############################################################################################################
h=4.135667516e-15
c=299792458

def deg2rad(angle_deg):
    angle_rad=angle_deg*np.pi/180
    return angle_rad

def rad2deg(angle_rad):
    angle_deg=angle_rad*180/np.pi
    return angle_deg
    
def eV2Lambda(eV):
    """ 
    Converts energy (eV) into wavelenght (Angstrom)
    """

    Angstrom = h*c/eV*1e9*10
    return Angstrom

def Lambda2eV(Angstrom):
    """ 
    Converts wavelenghth (Angstrom) into energy (eV)
    """

    eV = h*c/Angstrom*1e9*10
    return eV

##############################################################################################################
##############################             Kappa to Euler Conversion        ##############################
##############################################################################################################
def Kappa2Fourc(ang1,ang2,ang3,conversion,k_arm=50):

    a,b,c=0,0,0
    if conversion=='Kappa':
        a,b,c=EtoK(ang1,ang2,ang3,k_arm)
        print(("\n"+"th  = "+str(ang1)))
        print(("chi = "+str(ang2)))
        print(("phi = "+str(ang3)))
        print("~~~~~~~~")
        print(("kth  = "+str(a)))
        print(("kap  = "+str(b)))
        print(("kphi = "+str(c)+"\n"))
    elif    conversion=='fourc':
        a,b,c=KtoE(ang1,ang2,ang3,k_arm)
        print(("\n"+"kth  = "+str(ang1)))
        print(("kap  = "+str(ang2)))
        print(("kphi = "+str(ang3)))
        print("~~~~~~~~")
        print(("th  = "+str(a)))
        print(("chi = "+str(b)))
        print(("phi = "+str(c)+"\n"))
    else:
        print('2nd argument invalid; please choose one of the following:')
        print('"Kappa" or  "fourc"')
    return a,b,c



def EtoK(e_theta,e_chi,e_phi,k_arm=50):
    conv = np.pi/180.0
    kth_offset=caget('29idKappa:userCalcOut1.G')
    if(abs(e_chi)> 2.0*k_arm):
        print("Chi should be no more than twice the Kappa arm offset angle.")
        kth,kap,kphi=0,0,0
    else:
        print(("Calculating Kappa angles using kth0 = "+str(kth_offset)))
        k_ang = k_arm*conv
        delta = np.asin( - np.tan(e_chi*conv/2.0) / np.tan(k_ang) )
        k_theta = e_theta*conv - delta
        k_kappa = 2.0 * np.asin( np.sin(e_chi*conv/2.0) / np.sin(k_ang))
        k_phi   = e_phi*conv - delta
        #print k_theta, k_kappa,k_phi
        kth = rounder(k_theta)-(57.045-kth_offset)
        kap = rounder(k_kappa)
        kphi  = rounder(k_phi)
        #print delta
    return (kth,kap,kphi)




def KtoE(k_theta,k_kappa,k_phi,k_arm=50):
    conv = np.pi/180.0
    kth_offset=caget('29idKappa:userCalcOut1.G')
    print(("Calculating Euler angles using kth0 = "+str(kth_offset)))
    k_ang = k_arm*conv
    delta = np.atan( np.tan(k_kappa*conv/2.0) * np.cos(k_ang) )
    e_theta = k_theta*conv - delta
    e_chi   = 2.0 * np.asin( np.sin(k_kappa*conv/2.0) * np.sin(k_ang) )
    e_phi   = k_phi*conv - delta
    #print round(e_theta,1),round(e_phi,1),round(e_chi,1)
    theta = rounder(e_theta)+(57.045-kth_offset)
    chi   = rounder(e_chi)        # convert from rad to deg
    phi   = rounder(e_phi)
    #print round(delta,1)
    return (theta,chi,phi)


def rounder(val):        # convert from rad to deg
    conv = np.pi/180.0
    roundVal=round(1000.0*val/conv)/1000.0
    return roundVal
