from os import listdir,mkdir,chmod
from os.path import join, isfile, exists
import datetime
from time import sleep
import re

from epics import caget, caput


##############################################################################################################
################################             Standard Paths              ##############################
##############################################################################################################


path_User_Folders='/home/beams/29IDUSER/Documents/User_Folders/'

def path_dserv(folder,run,user_name):
    """
    Returns the path to a user folder
            dataFolder='/net/s29data/export/data_29id'+folder+'/'+run+'/'+user_name

    previously: _userDataFolder
    """
                    
    dataFolder='/net/s29data/export/data_29id'+folder+'/'+run+'/'+user_name
    return dataFolder







##############################################################################################################
################################                     Folders                    ##############################
##############################################################################################################
def check_run():
    """
    gets the current date and determines the run number from that
    run = year_cycle
        cycle 1 => Jan - April
        cycle 2 => May - September
        cycle 1 => October - December

    previously: Check_run
    """
    todays_date = datetime.date.today()

    date1 = ( 1, 1)   # start date run_1
    date2 = ( 5, 1)   # start date run_2
    date3 = (10, 1)   # start date run_3

    datex = (todays_date.month,todays_date.day)

    if date1 <= datex < date2:
        run=str(todays_date.year)+'_1'
    elif date2 <= datex < date3:
        run=str(todays_date.year)+'_2'
    else:
        run=str(todays_date.year)+'_3'

    return(run)

def make_ftp(folder,run,user_name):
    """
    Creates the folders on kip (ftp server) and prints what is needed for the cronjob

    folder = 'c' /'d'

    Previously: was part of Make_DataFolder
    """
    crontime={
        'mda2ascii':'0,30 * * * * ',
        'chmod':'1,31 * * * * ',
        'data_other':'2,32 * * * * ',
        'notebook':'*/3 * * * * ',
        }

    #making the crontab text
    print('-------------------------------------------------------------')
    #mda2ascii
    MyPath_kip_run='/net/kip/sftp/pub/29id'+folder+'ftp/files/'+run+'/'
    MyPath_kip='/net/kip/sftp/pub/29id'+folder+'ftp/files/'+run+'/'+user_name+'/'
    cmd_mda2ascii=crontime['mda2ascii']+' /net/s29dserv/APSshare/bin/mda2ascii -d '+MyPath_kip+'ascii '+MyPath_kip+'mda/*.mda'
    print(cmd_mda2ascii)
    #chmode
    cmd_chmod=crontime['chmod']+' chmod 664  '+MyPath_kip+'ascii/*.asc'
    print(cmd_chmod)
    #notebooks
    cmd_notebook=crontime['notebook']+' /usr/bin/rsync -av --exclude=core /home/beams22/29IDUSER/Documents/User_Folders/'+user_name+'/* kip:'+MyPath_kip+'notebook >  /home/beams22/29ID/cronfiles/cptoftp-currrun-d-User.log 2>&1'
    print(cmd_notebook)
    print('-------------------------------------------------------------\n\n')
    
    #making folders 
    print("\n\n")
    print(MyPath_kip)
    print(MyPath_kip+"ascii")
    if not (exists(MyPath_kip_run)):
        mkdir(MyPath_kip_run)
        chmod(MyPath_kip_run, 0o775)
    if not (exists(MyPath_kip)):
        mkdir(MyPath_kip)
        chmod(MyPath_kip, 0o775)
    if not (exists(MyPath_kip+"ascii")):
        mkdir(MyPath_kip+'ascii')
        chmod(MyPath_kip+'ascii', 0o775)
    if not (exists(MyPath_kip+"notebook")):
        mkdir(MyPath_kip+"notebook")
        chmod(MyPath_kip+"notebook", 0o775)
    
def make_user_folders(run,folder,user_name,endstation_name,ftp=False):
    """
    if user_name is not a folder in path_User_Folders, then it creates a new folder
    
    if ftp = True / False (default) creates the folders on kip (ftp server) and modifies the cronjob
    
    previously: Make_DataFolder
    """
    
    if (folder == 'c'or folder == 'd'):
       if ftp:
            make_ftp(folder,run,user_name)
    else:
            print("To create ftp folders & update contrab, you need to run the following as 29id:")
            print("\tFolder_"+str(endstation_name)+"('"+str(run)+"','"+str(user_name)+"',ftp=True)") 

        #create user folder for saving log files
    path_user=join(path_User_Folders,user_name)
    user_name = "/"+user_name
    if not (exists(path_user)):
        mkdir(path_user)


def folder_mda(run,folder,user_name,file_prefix,ioc,verbose=True):
    """
    sets the folder, prefix and file number for the scan record
    ioc is the full ioc, e.g. '29idb', or '29idARPES'

    For Staff: folder='b', user_name=''
    For ARPES: folder ='c'
    For Kappa: folder = 'd'

    mda_path="/net/s29data/export/data_29id"+folder+"/"+run+"/"+user_name+"mda"
    """
    
    if user_name == 'Staff':
        user_name=""
    else:
        user_name=user_name+'/'

    mda_path="/net/s29data/export/data_29id"+folder+"/"+run+"/"+user_name+"mda"
    print("\nMDA folder: " + mda_path)
    if not (exists(mda_path)):
        mkdir(mda_path)
        FileNumber=1
    else:
        FileNumber=get_next_fileNumber(mda_path,file_prefix)

    if ioc in ['29idb','29idc','29idd']: #VME iocs
        caput(ioc+"saveData_fileSystem","//s29data/export/data_29id"+folder+"/"+run)
        sleep(0.25)
        caput(ioc+"saveData_subDir",user_name+"mda")

    else: #soft iocs
        caput(ioc+"saveData_fileSystem","/net/s29data/export/data_29id"+folder+"/"+run)
        sleep(0.25) #needed so that it has time to write        
        caput(ioc+"saveData_subDir","/"+user_name+"mda")

    caput(ioc+"saveData_baseName",file_prefix)
    caput(ioc+"saveData_scanNumber",FileNumber)

    print("\nioc set to:", ioc)
    sleep(5)

    if verbose:
        SaveStatus=caget(ioc+'saveData_status',as_string=True)
        SaveMessage=caget(ioc+'saveData_message',as_string=True)
        print("\nSave Status "+ioc+": "+SaveStatus+" - "+SaveMessage)
       

def folder_SPEC(run,folder,user_name):
    if folder == "b":
        user_name = "Staff"
    else:
        user_name = user_name+"/"
    MyPath="/home/beams22/29IDUSER/spec/data/"+run+"/"+user_name
    print("\nSPEC folder: " + MyPath)
    print("You will need to create folder and set up the path manually in SPEC:")
    print("    cd "+MyPath)
    print("    newfile FileName")
    print("To start SPEC fresh: ./bin/kappa29ID -f")
    #if not (exists(MyPath)):
    #    mkdir(MyPath)


def folder_MPA(run,folder,user_name,file_prefix="mpa"):
    if folder == "b":
        windows_ioc = "X"
        user_name = ""
        windows_path = windows_ioc+':\\'+run+"\\mpa"
    else:
        windowsIOC = "Z"
        user_name = user_name+"/"
        windows_path=windows_ioc+':\\'+run+"\\"+user_name[:-1]+"\\mpa"
    mpa_path="/net/s29data/export/data_29id"+folder+"/"+run+"/"+user_name+"mpa"
    print("\nMPA folder: " + mpa_path)
    if not (exists(mpa_path)):
        mkdir(mpa_path)
        FileNumber=1
    else:
        FileNumber=get_next_fileNumber(mpa_path,file_prefix)
    caput("29iddMPA:det1:MPAFilename",windows_ioc+":/"+run+"/"+user_name+"mpa/mpa_")

    save_plugin='TIFF1'
    caput("29iddMPA:"+save_plugin+":FilePath",windows_path)
    print("\nMPA folder on Crabby: "+windows_path)
    caput("29iddMPA:"+save_plugin+":FileName",file_prefix)
    caput("29iddMPA:"+save_plugin+":FileNumber",FileNumber)

def _filename_key(filename):
    return (len(filename), filename)

def get_next_fileNumber(data_dir, file_prefix,**kwargs):
    """
    gets the next file number for the pattern 
    data_dir/file_prefix_filenum
    
    kwargs:
        verbose = False (default); if True then prints next file number
        debug = False (default)
    Previously: getNextFileNumber
    """
    kwargs.setdefault("verbose",False)
    kwargs.setdefault("debug",False)

    
    onlyfiles = [f for f in listdir(data_dir) if isfile(join(data_dir, f)) and f[:len(file_prefix)] == file_prefix]
    sortedfiles = sorted(onlyfiles, key=_filename_key)
    pattern = re.compile('(.*)_(.*)\.(.*)')
    try:
        lastFile = sortedfiles[-1]
    except IndexError as errObj:
        nextFileNumber = 1
        if kwargs["debug"]:
            print("Data directory = ", data_dir)
            print("File prefix =", file_prefix)
            print("File number =", None)
            print("File extension =", "TBD")
            print("Next File number =", nextFileNumber)
    else:
        matchObj = pattern.match(lastFile)
        nextFileNumber = int(matchObj.group(2)) + 1
        if kwargs["debug"]:
            print("Data directory = ", data_dir)
            print("File prefix =", matchObj.group(1))
            print("File number =", matchObj.group(2))
            print("File extension =", matchObj.group(3))
            print("Next File number =", nextFileNumber)
    if kwargs["verbose"] == True:
        print("Next File Number: ",nextFileNumber)
    return nextFileNumber







