"""
General functions for dealing with Area Detectors and Cameras at 29ID

work in progress need to redo
"""
##############################################################################################################
##############################     General Area Detector            ##############################
##############################################################################################################
import datetime
import re
from os import listdir,mkdir,chown,system,chmod
from os.path import join, isfile, exists, dirname
from time import sleep

from epics import caget, caput
from iexcode.instruments.files_and_folders import get_next_fileNumber


def AD_CurrentDirectory(ADplugin):
    """
    returns the current directory for area detector SavePlugin
    handles both Winodws and linux IOCs
    ADplugin = "29idc_ps1:TIFF1:"; $(P)$(SavePlugin)
    """
    SubDir=caget(ADplugin+"FilePath",as_string=True)
    if SubDir[0] == 'X':
        Dir='/net/s29data/export/data_29idb/'
        SubDir=SubDir.split('\\')[1:]
    elif SubDir[0] == 'Y':
        Dir='/net/s29data/export/data_29idc/'
        SubDir=SubDir.split('\\')[1:]
    elif SubDir[0] == 'Z':
        Dir='/net/s29data/export/data_29idd/'    
        SubDir=SubDir.split('\\')[1:]
    else: 
        Dir = SubDir
        SubDir=[]
    FilePath=join(Dir,*SubDir,'')
    return FilePath 

def AD_prefix(ADplugin):
    """
    returns the prefix for AreaDetector plugin based on ADplugin 
    """
    prefix = caget(ADplugin+"FileName_RBV",as_string=True)
    return prefix

def AD_EnableStats(ADplugin):
    """
    Enabling the statistics in an AreaDector
    ADplugin = "29idc_ps1:Stats1:"; (ADplugin=$(P)$(StatsPlugin))
    """
    caput(ADplugin+"EnableCallbacks","Enable")
    caput(ADplugin+"ComputeStatistics","Yes")
    caput(ADplugin+"ComputeCentroid","Yes")
    

def AD_SaveFileSetup(ADplugin,mda,**kwargs):
    """    
    ADplugin = "29id_ps1:TIFF1:" which IOC and which filesaving plugin
            (ADplugin=$(P)$(SavePlugin))
    uses to get the current MDA directory and then set the path to one up + /dtype
    MDA_CurrentDirectory(scanIOC=None)
    
    **kwargs (defaults)
        scanIOC = BL_ioc()
        userpath = extracted from ScanRecord unless specified
        subfolder = taken from ADplugin unless specified
            filepath = userpath/subfolder
        
        prefix = default same as subfolder
        ext = file extension is extracted for ADplugin unless specified
            (TIFF -> tif, HDF -> h5, ...)
        FileTemplate="%s%s_%4.4d."+ext; format for filename first %s = filepath, second %s = prefix
        
    """
    kwargs.setdefault("userpath",dirname(dirname(mda.filepath)))
    kwargs.setdefault("subfolder",ADplugin.split(":")[-2][:-1])

    kwargs.setdefault("prefix",ADplugin.split(":")[-2][:-1])
    extDict={"TIFF":".tif","HDF":"h5"}
    kwargs.setdefault("ext",ADplugin.split(":")[-2][:-1])
    kwargs.setdefault("FileTemplate","%s%s_%4.4d."+kwargs["ext"])
    
    kwargs.setdefault("debug",False)
    
    if kwargs['debug']:
        print("kwargs: ",kwargs)

    fpath=join(kwargs['userpath'],kwargs['subfolder'],'')
    print("\nFolder: " + fpath)
    if not (exists(fpath)):
        fileNumber=1
    else:
        fileNumber=get_next_fileNumber(fpath,kwargs["prefix"])
    print("NextFile: "+str(fileNumber))
    caput(ADplugin+"CreateDirectory",-1) #allows IOC to create directories    
    caput(ADplugin+"FilePath",fpath)
    caput(ADplugin+"FileName",kwargs["prefix"])
    caput(ADplugin+"FileNumber",fileNumber)
    
    #setup AD
    caput(ADplugin+"FileTemplate",kwargs["FileTemplate"])
    caput(ADplugin+"AutoIncrement","Yes")
    caput(ADplugin+"AutoSave","Yes")
    
def AD_CurrentPrefix(ADplugin):
    """
    returns the prefix (without _) for area detector SavePlugin
    ADplugin = "29id_ps1:TIFF1:"; $(P)$(SavePlugin)
    """
    Prefix=caget(ADplugin+'FileName',as_string=True)
    return Prefix
   
def AD_CurrentRun(ADplugin):
    """
    returns the curent run specified in the filepath for area detector SavePlugin
    ADplugin = "29id_ps1:TIFF1:"; $(P)$(SavePlugin)
    """
    fpath=caget(ADplugin+"FilePath",as_string=True)
    current_run=re.findall("\d\d\d\d_\d", fpath)[0]
    return current_run
   
def AD_CurrentUser(ADplugin):
    """
    returns the curent user specified in the filepath for area detector SavePlugin
    ADplugin = "29id_ps1:TIFF1:"; $(P)$(SavePlugin)
    """
    folder_name = ADplugin.split(":")[1]
    folder_name[:-1]
    SubDir=caget(ADplugin+":FilePath",as_string=True)
    if SubDir[0] == 'X':
        current_user='Staff'
    elif SubDir[0] == 'Y':
        m=SubDir.find(AD_CurrentRun(ADplugin))
        n=SubDir.find(folder_name)
        current_user=SubDir[m+7:n]
    else: current_user=None
    return current_user

def AD_DoneSingleSave(ADplugin,**kwargs):
    """
    sets and AD up ready to save images
        Acquire -> Done
        ImageMode -> Single
        Save -> Enable
        e.g. ADplugin="29id_ps2:TIFF1:"
        
    **kwargs:
        P=first part of ADplugin if not specified
        R="cam1:"; other AD have "det1:"
    """
    kwargs.setdefault("P",ADplugin.split(":")[0]+":")
    kwargs.setdefault("R","cam1:")
    AD_Done(ADplugin,**kwargs)
    caput(kwargs["P"]+kwargs["R"]+"ImageMode","Single");sleep(.5)
    caput(ADplugin+"EnableCallbacks","Enable");sleep(.1)
        
def AD_FreeRun(ADplugin,**kwargs):
    """
    sets and AD to disable saving and free run
        Saving -> Disable
        Acquire -> Done
        ImageMode -> Single

        e.g. ADplugin="29id_ps2:TIFF1:"
        
    **kwargs:
        P=first part of ADplugin if not specified
        R="cam1:"; other AD have "det1:"
    """
    kwargs.setdefault("P",ADplugin.split(":")[0]+":")
    kwargs.setdefault("R","cam1:")
    AD_Done(ADplugin,**kwargs)
    caput(ADplugin+"EnableCallbacks","Disable");sleep(.1)
    caput(kwargs["P"]+kwargs["R"]+"ImageMode","Continuous");sleep(.1)
    caput(kwargs["P"]+kwargs["R"]+"Acquire","Acquire");sleep(.1)

def AD_Done(ADplugin,**kwargs):
    """
    sets acquisition to Done
    **kwargs:
        P=first part of ADplugin if not specified
        R="cam1:"; other AD have "det1:"
    """
    kwargs.setdefault("P",ADplugin.split(":")[0]+":")
    kwargs.setdefault("R","cam1:")
    caput(kwargs["P"]+kwargs["R"]+"Acquire","Done",wait=True,timeout=5*60)
    sleep(0.1)

def AD_snap(ADplugin,**kwargs):
    """
    takes an image and save the image for ADplugin
        e.g. ADplugin="29id_ps2:TIFF1:"
        
    use AD_SaveFileSetup to set filepath, prefix etc.
    use AD_CurrentDirectory to see current directory
    use AD_prefix to see current prefix
    
    **kwargs:
        P=first part of ADplugin if not specified
        R="cam1:"; other AD have "det1:"
        
        ExposureTime: changes both the exposure time and the acquire time for the snapshot
                      resets after acquisition
        FreeRun: True => disable setting and go back to continuous acquision 
                 False => leave saving enabled and camera in single acquision
    """
    kwargs.setdefault("P",ADplugin.split(":")[0]+":")
    kwargs.setdefault("R","cam1:")
    kwargs.setdefault("FreeRun",True)

    expT=caget(kwargs["P"]+kwargs["R"]+"AcquireTime_RBV")
    acqT=caget(kwargs["P"]+kwargs["R"]+"AcquirePeriod_RBV")
    
    AD_DoneSingleSave(ADplugin,**kwargs)
    
    if "ExposureTime" in kwargs:
        caput(kwargs["P"]+kwargs["R"]+"AcquireTime",kwargs["ExposureTime"])
        caput(kwargs["P"]+kwargs["R"]+"AcquireTime",kwargs["ExposureTime"]+.01)
        
    caput(kwargs["P"]+kwargs["R"]+"Acquire","Acquire",wait=True,timeout=5*60)
    
    if "ExposureTime" in kwargs:
        caput(kwargs["P"]+kwargs["R"]+"AcquireTime",expT)
        caput(kwargs["P"]+kwargs["R"]+"AcquireTime",acqT) 

    if kwargs["FreeRun"]:
        sleep(.1)
        AD_FreeRun(ADplugin,**kwargs)
        


def AD_ScanTrigger(ADplugin,mda,**kwargs):
    """
    Add Triggering of AreaDetector to scanIOC
    ADplugin = "29idc_ps1:TIFF1:" (ADplugin=$(P)$(SavePlugin))
    
    **kwargs
        scanIOC = "29id"+BL_ioc() if not specified
        scanDIM = 1
        P=first part of ADplugin if not specified
        R="cam1:"; other AD have "det1"
        detTrig = 2; detectorTrigger number
    """
    kwargs.setdefault("scanDIM",1)
    kwargs.setdefault("P",ADplugin.split(":")[0]+":")
    kwargs.setdefault("R","cam1:")
    kwargs.setdefault("detTrig",2)
    
    scanPV=mda.ioc+"scan"+str(kwargs["scanDIM"])    
    trigger=".T"+str(kwargs["detTrig"])+"PV"
    caput(scanPV+trigger,kwargs["P"]+kwargs["R"]+"Acquire",wait=True,timeout=5*60)
    
def ADplugin_ScanSetup(ADplugin,mda, **kwargs):
    """
    stop the acquisition, puts in ImageMode=Single
    enables saving
    add to detector trigger
    Does not press go
    
    ADplugin = "29idc_ps1:TIFF1:"; (ADplugin=$(P)$(SavePlugin))
    **kwargs
        # AD_ScanTrigger
        scanIOC = "29id"+BL_ioc() if not specified
        scanDIM = 1
        P=first part of ADplugin if not specified
        R="cam1:"; other AD have "det1:"
        detTrig = 2; detectorTrigger number
        
        # AD_SaveFileSetup
        filepath=userpath (from BL_ioc scanRecord)+"/dtype"
         (e.g. filepath="/net/s29data/export/data_29id"+folder+"/"+run+"/"+userName+"/"+df)
        dtype = taken from ADplugin
        FileTemplate="%s%s_%4.4d."+dtype; format for filename first %s = filepath, second %s = prefix
        prefix = dtype by default
    
    """
    #from AD_ScanTrigger
    kwargs.setdefault("P",ADplugin.split(":")[0]+":")
    kwargs.setdefault("R","cam1:")

    AD_DoneSingleSave(ADplugin,**kwargs)
    
    AD_SaveFileSetup(ADplugin,**kwargs)
    AD_ScanTrigger(ADplugin, **kwargs)
    trigger=".T"+str(kwargs["detTrig"])+"PV"
    scanPV=mda.ioc+"scan"+str(kwargs["scanDIM"])  
    print("WARNING: you need to need to disable saving and clear the trigger by hand after the scan")
    print("\tAD_FreeRun("+ADplugin+"); caput("+scanPV+trigger+",'')")

def AD_ROI_setup(AD,ROInum,xcenter=500,ycenter=500,xsize=50,ysize=50,binX=1,binY=1):  
    """
    AD = "29id_ps4"
    AD = "29iddMPA"
    """
    # roiNUM=1  MPA_ROI_SetUp(535,539,50,50)  center of MCP
    
    ADplugin=AD+':ROI'+str(ROInum)+':'
    xstart=xcenter-xsize/2.0
    ystart=ycenter-ysize/2.0
    caput(ADplugin+'MinX',xstart)
    caput(ADplugin+'MinY',ystart)
    caput(ADplugin+'SizeX',xsize)
    caput(ADplugin+'SizeY',ysize)
    caput(ADplugin+'BinX',binX)
    caput(ADplugin+'BinY',binY)
    caput(ADplugin+'EnableCallbacks','Enable')
    print(ADplugin+' - '+caget(ADplugin+'EnableCallbacks_RBV',as_string=True))
    #MPA_ROI_Stats(roiNUM)
    
def AD_OVER_SetUp(AD,ROInum,OVERnum,linewidth=5,shape='Rectangle'):
    """
    AD = "29id_ps4"
    AD = "29iddMPA"
    shape= 'Cross', 'Rectangle', 'Ellipse','Text'
    """
    OVER1=AD+":Over1:"+str(OVERnum)+":"
    ROI=AD+":ROI"+str(ROInum)+":"
    
    caput(ROI+'EnableCallbacks','Enable')
    caput(OVER1+"Name","ROI"+str(ROInum))
    caput(OVER1+"Shape",shape)
    caput(OVER1+"Red",0)
    caput(OVER1+"Green",255)
    caput(OVER1+"Blue",0)
    caput(OVER1+'WidthX',linewidth)
    caput(OVER1+'WidthY',linewidth)
    
    caput(OVER1+"PositionXLink.DOL",ROI+"MinX_RBV CP")
    caput(OVER1+"SizeXLink.DOL",ROI+"SizeX_RBV CP")
    caput(OVER1+"PositionYLink.DOL",ROI+"MinY_RBV CP")
    caput(OVER1+"SizeYLink.DOL",ROI+"SizeY_RBV CP")
   
    caput(OVER1+"Use","Yes")

    
def AD_OverLayCenter(x,y,AD,OverLay=1,Num=1):
    """
    Sets CenterX and CenterY for  AD:Over(OverLay):Num:
    eg. 29id_ps2:Over1:1: AD='29id_ps2',Overlay=1, Num=1
    """
    caput(AD+":Over"+str(OverLay)+":"+str(Num)+":CenterX",x)
    caput(AD+":Over"+str(OverLay)+":"+str(Num)+":CenterY",y)
    
def AD_OverLayCenter_get(AD,OverLay=1,Num=1):
    """
    Sets CenterX and CenterY for  AD:Over(OverLay):Num:
    eg. 29id_ps2:Over1:1: AD='29id_ps2',Overlay=1, Num=1
    """
    print('x = '+str(caget(AD+":Over"+str(OverLay)+":"+str(Num)+":CenterX")))
    print('y = '+str(caget(AD+":Over"+str(OverLay)+":"+str(Num)+":CenterY")))





