from epics import caput,caget

def _encoder_dictionary():
    """
    returns a dictionary with the encoder pvs
    encoder_dictionary_entry(name) = encoder_ioc, motor_ioc, encoder_list

    """
    d={
        'slit2B':("29idMini1:","29idb:",[13,14,15,16]),
        'slit3D':("29idMini2:","29idb:",[26,27]),
        'ARPES':("29idARPES:","29idARPES:",[1,2,3,4]),
    }
    return d

def encoder_name_list():
    d = _encoder_dictionary()
    for key in d.keys():
        print(key)

def encoder_sync(name):
    """
    sync all the encoders for name
    """
    encoder_ioc, motor_ioc, encoder_list = _encoder_dictionary()[name]
    for encoder_num in encoder_list:
        pv = motor_ioc+"m"+str(encoder_num)+".SYNC"
        caput(pv,1)

def encoders_slit2B_reset_zero():
    """
    Resetting Slit 2B encoders to 0 for the current.
    Slit size need to be set to 0.

    Previously: Reset_Slit2B_Encoders
    """
    name='slit2B'
    encoder_sync(name)  
    encoder_ioc, motor_ioc, encoder_list = _encoder_dictionary()[name]

    print('\nCurrent Position:')
    for e_num in encoder_list:
        pv = encoder_ioc+'e'+(e_num)+'Pos'
        print('e'+(e_num)+' = '+str(caget(pv)))
    
    print('\nSetting all Offsets to 0:')
    for e_num in encoder_list:
        pv = encoder_ioc+'e'+(e_num)+'Pos.EOFF'
        caput(pv,0)
    
    print('\nCurrent Position:')
    for e_num in encoder_list:
        pv = encoder_ioc+'e'+(e_num)+'Pos'
        print('e'+(e_num)+' = '+str(caget(pv)))

    print('\nSetting back Offsets:')
    for e_num in encoder_list:
        pos_pv = encoder_ioc+'e'+(e_num)+'Pos' 
        offset_pv = encoder_ioc+'e'+(e_num)+'Pos.EOFF' 
        caput(offset_pv,-caget(pos_pv))

    print('\nCurrent Position:')
    for e_num in encoder_list:
        pv = encoder_ioc+'e'+(e_num)+'Pos'
        print('e'+(e_num)+' = '+str(caget(pv)))

    encoder_sync(name)
    print('sync '+name)
