from epics import caget, caput
##############################################################################################################
###########################            String Sequences       ######################
##############################################################################################################
def userStringSeq_pvs(ioc, seq_num):     
    """
    returns the proc pv for a given userStringSeq
    userStringSeq, proc_pv  

    Previously: BeforeScan_StrSeq, AfterScan_StrSeq
    """
    userStringSeq_pv = ioc+"userStringSeq"+str(seq_num)
    userStringSeq_proc = userStringSeq_pv+".PROC"
    return userStringSeq_pv, userStringSeq_proc


def userStringSeq_clear(ioc,seq_num):
    """
    clears the nth suserStringSeq in the specified ioc    
    
    Previously: ClearStringSeq
    """
    pv=ioc+"userStringSeq"+str(seq_num)
    caput(pv+".DESC","")
    for i in range(1,10):
        caput(pv+".STR"+str(i),"")  
        caput(pv+".LNK"+str(i),"")
        caput(pv+".DOL"+str(i),"")
        caput(pv+".DO"+str(i),0.0)
        caput(pv+".DLY"+str(i),0.0)
        caput(pv+".WAIT"+str(i),"NoWait")
    caput(pv+".STRA","")
    caput(pv+".LNKA","")
    caput(pv+".DOLA","")
    caput(pv+".DOA",0.0)
    caput(pv+".DLYA",0.0)
    caput(pv+".WAITA","NoWait")
    caput(pv+".FLNK","")
    return pv

##############################################################################################################
###########################      User CalcsOut         ######################
##############################################################################################################
def userCalcOut_clear(ioc,calcOut_num):
    """
    clears the nth userCalcOut in the specified ioc

    Previously:ClearCalcOut
    """
    pv=ioc+"userCalcOut"+str(calcOut_num)
    caput(pv+".DESC","")

    for i in ["A","B","C","D","E","F","G","H","I","J","K","L"]:
        caput(pv+".INP"+i,"")
        caput(pv+"."+i,0)

    caput(pv+".CALC$","A")
    caput(pv+".OCAL$","A")
    caput(pv+".OUT","")
    caput(pv+".OOPT","On Change")
    return pv


##############################################################################################################
###########################      User Calcs, User  Average        User  Average         ######################
##############################################################################################################
def userAvg_clear(ioc,userAvg_num):
    """
    clears the nth userAvg in the specified ioc    
    
    Previously: ClearUserAvg
    """
    pv = ioc+"userAve"+str(userAvg_num)
    caput(pv+".DESC","")
    caput(pv+".INPB","")
    return pv

def userAvg_pvs(ioc, userAve_num):     
    """
    returns the proc pv for a given userStringSeq
    userStringSeq, proc_pv  

    """
    userAve_pv = ioc+"userAve"+str(userAve_num)
    userAve_proc = userAve_pv+".PROC"
    return userAve_pv, userAve_proc


def userAvg_trigger_strSeq(ioc,num,userAvg_num=8):
    """
    num = number of userAvg
    """
    userAvg_num = 8
    userAvg_clear(ioc,userAvg_num)
    str_pv="29id"+ioc+":userStringSeq"+str(userAvg_num)
    caput(str_pv+".DESC","Average Triggers_"+ioc)
    if num < 10:
        str_num = str(num)
    else:
        str_num = "A"
    for userAve_num in range(1,userAve_num+1,1):
        if userAve_num < 10:
            userAve_num = str(userAve_num)
        else:
            userAve_num = "A"
        userAve_pv, userAve_proc = userAvg_pvs(ioc, userAve_num)
        caput(str_pv+".LNK"+userAve_num,userAve_proc+" CA NMS")
        caput(str_pv+".WAIT"+userAve_num,"After"+str_num)


def UserAvg_setup(ioc,userAvg_num,det_num,average_pts,UserAvg_trigger_strSeq_pv,scan_dim=1):
    """
    kappa 
    userAv_num 1->10
    det_num 61 -> 70
    Previously:UserAvg
    """
    userAvg_pv = ioc+"userAve"+str(userAvg_num)
    scan_pv = ioc+"scan"+str(scan_dim)
    det_pv = scan_pv+".D"+str(det_num)+"PV"

    trigger_strSeq_pv,trigger_strSeq_proc = userAvg_trigger_strSeq(ioc)

    if average_pts > 0:
        caput(scan_pv+".T2PV",trigger_strSeq_proc)     #set all Avg as Det
        caput(userAvg_pv+".A",average_pts)
        caput(det_pv,userAvg_pv+".VAL")
        print("User Average enabled")
    elif average_pts == 0:
        caput(scan_pv+".T2PV","")
        caput(det_pv,"")
        print("User Average disabled")


def userAvg_setup(ioc,pv,userAvg_num,name,average_pts=10,mode="ONE-SHOT"):
    """
    
    Previously: UserAvg_PV
    """
    trigger_strSeq_pv,trigger_strSeq_proc = userAvg_trigger_strSeq(ioc)
    
    # Fill User Average:
    userAvg_clear(ioc,userAvg_num)
    userAve_pv, userAve_proc = userAvg_pvs(ioc, userAvg_num)

    caput(userAve_pv+".SCAN","Passive")
    caput(userAve_pv+".INPB",trigger_strSeq_pv+" CP NMS")
    caput(userAve_pv+".A",average_pts)
    caput(userAve_pv+".PREC",9)
    caput(userAve_pv+"_mode.VAL",mode)
    caput(userAve_pv+"_algorithm.VAL","AVERAGE")
    caput(userAve_pv+".DESC",name)    


def userAvg_trigger_strSeq(ioc,num, seq_num=8):
    """
    
    Previously: UserAvg_Trigger_StrSeq
    """
    userStringSeq_clear(ioc,seq_num)
    trigger_strSeq_pv,trigger_strSeq_proc = userStringSeq_pvs(ioc, seq_num)
    caput(trigger_strSeq_pv+".DESC",ioc+" Average Triggers")

    if num < 10:
        str_num=str(num)
    else:
        str_num="A"
    for i in range(1,num+1,1):
        if i < 10:
            i=str(i)
        else:
            i="A"
        avg_pv="29id"+ioc+":userAve"+i
        caput(trigger_strSeq_pv+".LNK"+i,avg_pv+"_acquire.PROC CA NMS")
        caput(trigger_strSeq_pv+".WAIT"+i,"After"+str_num)

    return trigger_strSeq_pv,trigger_strSeq_proc