"""
useful functions

"""
import time
import datetime
import select
import sys
import ast
import os
import numpy as np
from numpy import inf

from epics import caget, caput

def dateandtime():
    return time.strftime("%a %d %b %Y %H:%M:%S",time.localtime())
 
def today(which='default'):
    dt=datetime.date.today()
    today_year = dt.year
    today_month = dt.month
    today_day = dt.day
    if which == 'slash':
        today_str=("{:02d}/{:02d}/{:04d}".format(today_month,today_day,today_year))
    elif which == 'int':
        today_str=int(("{:04d}{:02d}{:02d}".format(today_year,today_month,today_day)))
    else: # 'default':
        today_str=("{:04d}{:02d}{:02d}".format(today_year,today_month,today_day))
    return today_str 


def print_warning_message(message_string):
     print('\n==================================')
     print("WARNING: " +message_string+ " !!!!! ")
     print('==================================\n')


def wait_for_it(D,H,M):
    """
    D = how many days from now
    H,M = what time that day in 24h clock

    e.g.: if today is           Wed Nov 21 at 14:00
        WaitForIt(2,9,0) => Fri Nov 23 at  9:00
    """
    t = datetime.datetime.now()()
    day = datetime.timedelta(days=D)
    future = t + day
    returnTime = datetime.datetime(future.year, future.month, future.day, int(H), int(M))
    timeToWait = returnTime - t
    s=round(timeToWait.total_seconds(),1)
    m=round(timeToWait.total_seconds()/60.0,1)
    h=round(timeToWait.total_seconds()/3600.0,1)
    print("Now is:      "+str(t))
    print("Target date: "+str(returnTime))
    print("Sleeping for "+str(s)+" s = "+str(m)+" m = "+str(h)+" h")
    print("Waaaaaaaait for it...")
    time.sleep(timeToWait.total_seconds())
    print(dateandtime())

    
def input_timeout(question,t):
    print("You have "+str(t)+" seconds to answer!")
    i, o, e = select.select( [sys.stdin], [], [], t )
    if (i):
        print("You said", sys.stdin.readline().strip())
    else:
        print("You said nothing!")

def playsound(sound='FF'):
    """
    plays a sound when run
    'FF' Final Fantasy victory sound
    'ding' a subtle ding noise
    'hallelujah' hallelujah chorus
    """
    if sound == 'FF':
        sounds = '/home/beams/29IDUSER/Documents/User_Macros/Macros_29id/Sound_Files/VictoryFF.wav'
    elif sound == 'ding':
        sounds = '/home/beams/29IDUSER/Documents/User_Macros/Macros_29id/Sound_Files/ding.wav'
    elif sound == 'hallelujah':
        sounds = '/home/beams/29IDUSER/Documents/User_Macros/Macros_29id/Sound_Files/hallelujah.wav'
    os.system('aplay ' + sounds)

def range_up(start,end,step):
    while start <= end:
        yield start
        start += abs(step)

def range_down(start,end,step):
    while start >= end:
        yield start
        start -= abs(step)

def EgForLoop():
    for hv in range_down(2000,500,-500):
        print(hv)
    for hv in range_up(500,2000,500):
        print(hv)

def take_closest_value(my_list,my_number):
    """
    Given a list of integers, I want to find which number is the closest to a number x.
    
    Previously: TakeClosest
    """
    return min(my_list, key=lambda x:abs(x-my_number))

def read_dict(filename,**kwargs):
    """
    filename = name with extension of dictionary to read
        **kwargs:
            path = path to folder containing dictionary file (default: .IEX_dictionaries)
    """ 
    if 'path' in kwargs:
        path = kwargs['path']
    else:
        path = os.path.join(os.path.dirname(__file__),'IEX_dictionaries')
    fpath = os.path.join(path,filename)
    with open(fpath) as f:
        for c,line in enumerate(f.readlines()):
            if line[0] == '=':
                lastdate=line[8:16]
            lastline=line
        mydict=ast.literal_eval(lastline)
    return mydict

    
def make_table(start_stop_step_lists):
    """
    Creates and returns a np.array with values based on StartStopStepList
    StartStopStepList is a list of lists defining regions for a table array
              StartStopStepList[[start1,stop1,step1],[start1,stop1,step1],...]
    Automatically removes duplicates and sorts into ascending order
    if you want descending
               myarray=XAS_Table(StartStopStepLists)[::-1]

    Previously: Scan_MakeTable           
    """
    table_array=np.array([])
    if type(start_stop_step_lists) is not list:
        start = start_stop_step_lists[0]
        stop = start_stop_step_lists[1]
        step = start_stop_step_lists[2]
        j = start
        while j<=stop:
            table_array=np.append(table_array, j)
            j+=step
    else:
        for i in range(0,len(start_stop_step_lists)):
            start=start_stop_step_lists[i][0]
            stop=start_stop_step_lists[i][1]
            step=start_stop_step_lists[i][2]
            j=start
            while j<=stop:
                table_array=np.append(table_array, j)
                j+=step
    table_array=np.unique(table_array)#removing duplicate
    table_array=np.sort(table_array) #sort into ascending order    
    return table_array



def SendText(msg,which='user'):
    """
    
    which = 'user' or 'staff'
    
    Edit the following file with the correct email/phone number:
        [sleepy /home/beams22/29ID/bin] pvMailSomething        # for staff
        [sleepy /home/beams22/29IDUSER/bin] pvMailUser        # for user
        
    Use the following website to figure out carriers email:
        https://20somethingfinance.com/how-to-send-text-messages-sms-via-email-for-free/
    
    In a terminal, run the screen session:
        pvMailUser (or pvMailSomething)
        
    To kill the screen session:
        screen -ls                         #show list of screen session
        screen -r screen#              #reattached to a given screen session
        screen -XS screen# kill         #kill a given screen session
        

    """
    if which == 'staff':n='6'
    elif which == 'user':n='7'
    caput('29idb:userCalcOut'+n+'.DESC',msg)
    caput('29idb:userCalcOut'+n+'.A',0)
    time.sleep(1)
    caput('29idb:userCalcOut'+n+'.A',1)
    print('Sending text/email')


def AbortScript():
    """
    Example:
        while True:
                try:
                print 'do something'
                sleep(5)
                except KeyboardInterrupt:
                if AbortScript() == 'y':
                        break
    """
    try:
        print('\n\nWARNING: Do you want to abort?')
        print('Type y to ABORT, anything else to CONTINUE >')
        foo = input()
        return foo
    except KeyboardInterrupt as e:
        raise e

