from epics import caget, caput

import iexcode.instruments.cfg as iex

branch_GVs={
    'c':('10'),
    'd':('14'),
    }

def branch_valve_close():
    """
    closes the endstation valve for the current branch
    """
    GV = branch_GVs[iex.BL.branch]
    valve_close(GV, verbose=True)
    
def branch_valve_open():
    """
    closes the endstation valve for the current branch
    """
    GV = branch_GVs[iex.BL.branch]
    valve_open(GV, verbose=True)

    
def valve_close(GV, verbose=True):
    """
    closes the gate valve ('GV10', 'GV14'), EPS nomenclature

    Previously: Close_DValve,Close_DValve
    """
    pv = "29id:BLEPS:GV"+str(GV).zfill(2)+":CLOSE.VAL"
    status = valve_status(GV)
    if status == 'closed':
        message = "Gate valve: "+GV+" is already closed"
    elif status == 'open':
        caput(pv,1,wait=True,timeout=18000)
        message = "Closing gate valve: "+GV+"..."
    else:
        caput(pv,1,wait=True,timeout=18000)   
        message = "Gate valve: "+GV+" was in an error state, trying to close..."
    if verbose:
        print(message)

def valve_open(GV, verbose=True):
    """
    closes the gate valve ('GV10', 'GV14'), EPS nomenclature

    Previously: Close_DValve,Close_DValve
    """
    pv = "29id:BLEPS:GV"+str(GV).zfill(2)+":OPEN.VAL"
    status = valve_status(GV)
    if status == 'open':
        message = "Gate valve: "+GV+" is already open"
    elif status == 'closed':
        caput(pv,1,wait=True,timeout=18000)
        message = "Opening gate valve: "+GV+"..."
    else:
        caput(pv,1,wait=True,timeout=18000)   
        message = "Gate valve: "+GV+" was in an error state, trying to open..."
    if verbose:
        print(message)

def valve_status(GV,verbose=True):
    """
    returns the current status ('open','closed','error')
    """
    open_status = bool(caget("29id:BLEPS:GV"+str(GV).zfill(2)+":OPENED:STS"))
    closed_status = bool(caget("29id:BLEPS:GV"+str(GV).zfill(2)+":CLOSED:STS"))

    if open_status == True:
        if closed_status == False:
            status = 'open'
        else:
            status = 'error'

    if closed_status == True:
        if open_status == False:
            status = 'closed'
        else:
            status = 'error'
    
    return status