import matplotlib.pyplot as plt

from epics import caget,caput
from ..instruments.kappa import *
from ..instruments.Motors import *

from .quick_plot import fit_mda, mda_1D

##### Kappa Optimization scans #####
def Opt_d4(iteration,moveZ,graph='y',srs=None,start=-0.5,stop=0.5,step=0.04):
    tth_w=0.1
    if srs==None: det=21
    else: det=34
    z=caget('29idKappa:m4.RBV')
    current_det=caget('29idKappa:userStringSeq6.STR1',as_string=True)
    if current_det != 'd4':
        print('Switching detector to d4')
        tthdet.set('d4',move=False)
        mvtth(0)
    if moveZ is not None:
        mvz(z-1000)
    for i in range(iteration):
        scantth(start,stop,step,'relative')
        scannum = mda.lastFileNum()
        tth0=fit_mda(scannum,det,tth_w,'gauss',graph=graph)
        mvtth(tth0)
    mvz(z)
    return tth0,scannum

def Opt_z0(iteration,movetth,movekth,det='d4',srs=None,graph='y'):
    z_w=400
    current_det=caget('29idKappa:userStringSeq6.STR1',as_string=True)
    if current_det != det:
        print('Switching detector to '+det)
        tthdet.set(det,move=False)
    if det=='d3' and srs==None: det=20
    if det=='d3' and srs!=None: det=33
    if det=='d4' and srs==None: det=21
    if det=='d4' and srs!=None: det=34
    if movetth is not None:
        mvtth(movetth)  
    if movekth is not None:
        mvkth(movekth)
    if iteration>1:
        scanz(-2000,2000,250,'relative')
        scannum=mda.fileNum()
        z0=fit_mda(scannum,det,z_w,'erf',graph=graph)
        mvz(z0)
    scanz(-700,700,50,'relative')
    scannum = mda.lastFileNum()
    z0=fit_mda(scannum,det,z_w,'erf',graph=graph)
    mvz(z0)
    return z0,scannum

def Opt_kth(kth_0,theta,det='d4',srs=None,graph='y'):
    current_det=caget('29idKappa:userStringSeq6.STR1',as_string=True)
    if current_det != det:
        print('Switching detector to '+det)
        tthdet.set(det,move=False)
    if det == 'd4': 
        if srs==None: det=21
        else: det=34
        i=1
    elif det == 'd3': 
        if srs==None: det=20
        else: det=33
        i=5
    mvtth(theta*2)
    mvkth(kth_0+theta)
    scankth(-0.5*i,0.5*i,0.05*i,'relative')
    new_kth=fit_mda(mda.lastFileNum(),det,0.1,'gauss',graph=graph)
    mvkth(new_kth)
    scankth(-0.2*i,0.2*i,0.02*i,'relative')
    new_kth=fit_mda(mda.lastFileNum(),det,0.1,'gauss',graph=graph)
    mvkth(new_kth)
    scannum=mda.lastFileNum
    kth0=round(new_kth-theta,3)
    print('\nkth0 = ',kth0)
    print('To plot use:') 
    print('     fit_mda('+str(scannum)+',21,0.2,"gauss",mytitle=\'theta ='+str(theta)+'\')')
    print('If you are happy with that, you can set this value as kth0 using:')
    print('     kth0_set('+str(kth0)+')')

    return kth0,scannum

def plot_opt(opt_scan_list,energy_list,det,mytitle=''):
    fig,(ax1)=plt.subplots(1,1)
    fig.set_size_inches(5,4)

    for i,j in zip(opt_scan_list,energy_list):
        x,y,x_name,y_name=mda_1D(i,det,1,0)
        xdata = np.array(x)
        ydata = np.array(y)
        Imax=np.max(ydata)    
        ax1.plot(x,y/Imax,label=str(j)+" eV")
    ax1.set(ylabel='Intensity')
    ax1.set(xlabel=x_name)
    ax1.ticklabel_format(style='sci', axis='y', scilimits=(0,0))
    ax1.grid(color='lightgray', linestyle='-', linewidth=0.5)
    ax1.legend(bbox_to_anchor=[1.2, 1],ncol=2,shadow=True, title=mytitle, fancybox=True)    
